
/**
  Sample Java AWT application
    Philip Brown, 2013. phil@bolthole.com
       http://www.bolthole.com/java
  This is a sample skeleton for any java application.
  50% of getting started writing a java application, is duplicating stuff 
  that needs to be in pretty much any good program.
  
  So, here is a skeleton that is free to use as the start of
  your own Java Application.
  (It's geared for AWT, though!!)
  It goes above and beyond the initial stuff, because it has menus, and
  proper close-button handling, and an icon.
  
  Currently, this program requires this file, plus the following:
      ErrDialog.java
      About.java
      
  You can use, reuse, or do whatever you like with this code.
  I just ask that you leave the following two lines in:
  
  // Started from the app skeleton by Philip Brown
  //  http://www.bolthole.com/java
   
 * 
 */




import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;

public class JavaApp extends Frame implements 
  ActionListener, WindowListener 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	TextArea welcome; // This also functions as a status bar
	 
	About aboutwin;
	
	/**
	 * This is where the program really starts! 
	 * But it's only the "static" entry point. The real work happens
	 * when we create the main window for the program, JavaApp() 
	 */
	public static void main(String[] args) {
		JavaApp me=new JavaApp();
		me.setVisible(true);
	}

	void quit(){
		System.exit(0);
	}
	
	// Pops up a separate window with your message
	void popupmsg(String msg){
		ErrDialog d=new ErrDialog(this,msg);
		d.setVisible(true);
		d=null; // good for memory management.
	}
	void errquit(String errmsg){
		popupmsg(errmsg);
		quit();

	}
	
	// Prints messages out on "console", not the window.
	static void debug(String msg){
		System.err.println(msg);
	}
	
	
	// Set, or reset, the standard welcome message
	void setWelcome(){
		welcome.setText("This is a sample java AWT application\n"+
						"From http://www.bolthole.com/java");
	}
	
	// Having std_cursor==null, actually seems to be the best trick
	// to do, to "reset cursor to standard"
	Cursor std_cursor=null, busy_cursor;
	

	/**
	 * This makes almost all of the window areas. Actual stuff getting done, happens elsewhere.
	 */
	public JavaApp(){
		initIcon();
		
		busy_cursor=Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
		setCursor(busy_cursor); 
		// Dont think this actually works in here...
		// But it's a good thing to use, so I leave it here
		// as a reference for you.
		
		
		/* We now make all the sub-areas of the main window!
		 * I use the default "layout" here, but there are a whole bunch of others
		 * you might choose to use:
		 * GridLayout, FlowLayout, GridBagLayout....
		 * If you want to stick with AWT, get to know them and
		 * what they can do
		 */
		setLayout(new BorderLayout(10,10));
		
		welcome=new TextArea("",2,35,TextArea.SCROLLBARS_NONE);
		setWelcome();
		welcome.setEditable(false);
		welcome.setBackground(Color.green);
		add(welcome,BorderLayout.NORTH);
			
						
		Panel cp=new Panel();  // CenterPanel
		add(cp, BorderLayout.CENTER);
		TextArea centertext=new TextArea("This is a new application\n"+
			"Its main window is here\n"+
			"It can have lots of text\n"+
			"But maybe buttons or something would be more fun");
		
		cp.add(centertext);
		
		
		/* And now, some buttons at the bottom.
		 * But they can be anywhere you like. Just learn to use the
		 * layouts to place them where you want!
		 */
		Panel bpanel=new Panel(new FlowLayout());
		
		Button hi_b, quit_b;
		hi_b=new Button("Hi! Click me!");
		hi_b.setBackground(Color.green);
		hi_b.addActionListener(this);
		bpanel.add(hi_b);
		
		quit_b=new Button("Quit");
		quit_b.addActionListener(this);
		bpanel.add(quit_b);
		
		add(bpanel, BorderLayout.SOUTH);
		

		makeMenus();
		
		// This allows window-corner-clicks to close us.
		addWindowListener(this);

		 // This resets the cursor from "busy" to whatever	
		setCursor(std_cursor);	
		
		pack(); // makes everything organize inside the window

	}
	
	/**
	 * Right now, we just add one menu item: the "About" one,
	 * under the top level "Help". Add as many others as you like!
	 */
	void makeMenus(){
		MenuBar menubar = new MenuBar();
		
		Menu helpmenu= new Menu("Help");
		MenuItem about=new MenuItem("About");
		about.addActionListener(this);
		helpmenu.add(about);
		
		menubar.add(helpmenu);
		this.setMenuBar(menubar);
		
		// kindasorta menu-y. Handles the window corner close click.
		

	}
	// end standard constructor
	
		
	/**
	 * This handles clicks on Buttons, AND
	 * clicks on menu items
	 */
	public void actionPerformed(ActionEvent e){
		String command=e.getActionCommand();
		debug("actionPerformed: Got an event. Command="+command);
		
		if(command=="About"){
			if(aboutwin==null){
				aboutwin=new About();
			}
			aboutwin.setVisible(true);
			return;
		}
		if(command=="Quit"){
			quit();
		} 
		// Else... only one left!
		popupmsg("Thanks for clicking!");
	}
	
	
	// All of these, just so we can have windowClosing
	// which allows us to close gracefully if window frame is closed
    public void windowOpened(WindowEvent evt)   {}
    public void windowClosed(WindowEvent evt)   {}
    public void windowActivated(WindowEvent evt){}
    public void windowDeactivated(WindowEvent evt){}
    public void windowIconified(WindowEvent evt)  {}
    public void windowDeiconified(WindowEvent evt){}
    public void windowClosing(WindowEvent evt)
    {
            quit();
    }
    
    
    /* for older JDKs that dont have javax.imageio
     * 
     */
    Image oldImageRead(InputStream instr){
        Toolkit tk = Toolkit.getDefaultToolkit();
        Image returnval=null;
        byte myicon[];
        try
        {
        	myicon=new byte[instr.available()];
        	instr.read(myicon);
        	returnval = tk.createImage(myicon);

        } catch (java.io.IOException err)
        {
        	debug("error loading icon file:"+err.getMessage());
        }  	
        return returnval;
    }
    
    // Gives our program its own icon!! if "icon.png" is present in jarfile, anyway
    // Note: MUST BE CALLED BEFORE setVisible()
    // Note also that it only shows up when iconified, NOT when running
    void initIcon()
    {
        Image iconimage;
        String iconname="icon.png";
        // adapted from "java FAQ", from PVL
        InputStream instr = getClass().getResourceAsStream(iconname);
        if(instr==null){
        	debug("initIcon cannot get resource. Not in a JAR?");
        	return;
        }

        iconimage=oldImageRead(instr);

        if(iconimage==null)
        {
        	debug("icon image create FAILED?!");
        	return;
        } else
        {
        	debug("initIcon now trying to set "+iconname);
        	//debug("  width is "+iconimage.getWidth(this));
        	//debug("  height is "+iconimage.getHeight(this));
        }
        setIconImage(iconimage);
    }
	
	
}
